// Midibus Droplets - Jim Bumgardner 10/2/2011
//
// A recreation of my 'Hypnoticon' (circa 6/94)
//

import themidibus.*; //Import the library

MidiBus myBus; // The MidiBus

boolean midiThru = false;  // echo all received midi in messages to the midi out device

int  maxNotes = 128;

class Note {
  long   startTime;
  float  x;
  float  y;
  float  initRad;
  float  weight;
  float  kMinVelocity = 10;
  float  kMaxVelocity = 100;
  int    pitch;
  int    velocity;
  
  boolean  isOn = false;

  Note(int pitch) {
    this.pitch = pitch;
  }
  
  void init(int pitch, int velocity)
  {
    startTime = millis();
    this.pitch = pitch;
    this.velocity = velocity;
    this.x = random(width);
    this.y = random(height);
    this.initRad = 8 + velocity/128.0 * 20;
    this.weight = 2 + velocity/128.0 * 10;
    this.isOn = true;
  }
  
  void render()
  {
    if (!isOn)
      return;
    float elapsed = (millis() - startTime)*.001;
    float hueV = (this.pitch % 12) / 12.0;
    float lum = constrain(1-pow(elapsed/4,2), .05, 1);
    float sat = lum;
    float rad = initRad + elapsed*(kMinVelocity + (kMaxVelocity-kMinVelocity)*velocity/128.0);
    noFill();
    strokeWeight(weight);
    stroke(hueV,sat,lum);
    ellipse(x,y,rad*2,rad*2);
    // embossed shading
    stroke(hueV,sat,0,.5);
    ellipse(x+3,y+3,rad*2,rad*2);
  }
};


void setup() {
	size(800,600);
	background(0);

	MidiBus.list(); // List all available Midi devices on STDOUT. This will show each device's index and name.

        // MODIFY THIS FOR YOUR MIDI SETUP
	//                 Parent  In        Out
	//                   |     |          |
	myBus = new MidiBus(this, "Port A", "Port A"); // Create a new MidiBus with no input device and the default Java Sound Synthesizer as the output device.

        colorMode(HSB, 1);
        smooth();
        noFill();

        for (int i = 0; i < maxNotes; ++i) {
          notes[i] = new Note(i);
        }
}


Note[] notes = new Note[128];

void draw() {
    background(0);
    for (int i = 0; i < maxNotes; ++i) {
      notes[i].render();
    }
}

void noteOn(int channel, int pitch, int velocity) {
	// Receive a noteOn
	println("Note On : ch=" +channel+" pitch="+pitch+" velocity="+velocity);
        if (midiThru) {
          myBus.sendNoteOn(channel, pitch, velocity); // Send a Midi noteOn
        }
        notes[pitch].init(pitch, velocity);
}

void noteOff(int channel, int pitch, int velocity) {
	// Receive a noteOff
	// println("Note Off: ch=" +channel+" pitch="+pitch+" velocity="+velocity);
        if (midiThru) {
          myBus.sendNoteOff(channel, pitch, velocity); // Send a Midi noteOn
        }
        notes[pitch].isOn = false;
}

void controllerChange(int channel, int number, int value) {
	// Receive a controllerChange
	// println("Ctrl Ch : ch=" +channel+" number="+number+" value="+value);
        if (midiThru) {
          myBus.sendControllerChange(channel, number, value); // Send a controllerChange
        }
}


