// Midibus - basic io - modified from 'basic' example in Midibus examples folder.

import themidibus.*; //Import the library

MidiBus myBus; // The MidiBus

boolean autoPulse = false;
boolean midiThru = true;  // echo all received midi in messages to the midi out device

void setup() {
	size(400,400);
	background(0);

	MidiBus.list(); // List all available Midi devices on STDOUT. This will show each device's index and name.

	// Either you can
	//                   Parent In Out
	//                     |    |  |
	//myBus = new MidiBus(this, 0, 1); // Create a new MidiBus using the device index to select the Midi input and output devices respectively.

	// or you can ...
	//                   Parent         In                   Out
	//                     |            |                     |
	//myBus = new MidiBus(this, "IncomingDeviceName", "OutgoingDeviceName"); // Create a new MidiBus using the device names to select the Midi input and output devices respectively.

	// or for testing you could ...
	//                 Parent  In        Out
	//                   |     |          |
	myBus = new MidiBus(this, "Plug 1", "Java Sound Synthesizer"); // Create a new MidiBus with no input device and the default Java Sound Synthesizer as the output device.
}

void draw() {
        if (autoPulse) {
          int channel = 0;
          int pitch = 64;
          int velocity = 127;
          	
          myBus.sendNoteOn(channel, pitch, velocity); // Send a Midi noteOn
          delay(200);
          myBus.sendNoteOff(channel, pitch, velocity); // Send a Midi nodeOff
          
          int number = 0;
          int value = 90;
          	
          myBus.sendControllerChange(channel, number, value); // Send a controllerChange
          delay(2000);
        }
}

void noteOn(int channel, int pitch, int velocity) {
	// Receive a noteOn
	println("Note On : ch=" +channel+" pitch="+pitch+" velocity="+velocity);
        if (midiThru) {
          myBus.sendNoteOn(channel, pitch, velocity); // Send a Midi noteOn
        }
}

void noteOff(int channel, int pitch, int velocity) {
	// Receive a noteOff
	println("Note Off: ch=" +channel+" pitch="+pitch+" velocity="+velocity);
        if (midiThru) {
          myBus.sendNoteOff(channel, pitch, velocity); // Send a Midi noteOn
        }
}

void controllerChange(int channel, int number, int value) {
	// Receive a controllerChange
	println("Ctrl Ch : ch=" +channel+" number="+number+" value="+value);
        if (midiThru) {
          myBus.sendControllerChange(channel, number, value); // Send a controllerChange
        }
}
