// Random MIDI Notes - Jim Bumgardner
//

import themidibus.*;
MidiBus myBus;

boolean isMute = false;

int kMidiChannel = 0;

int   kLowestNoteNumber = 32;    // MIDI note number (0-127)
int   kHighestNoteNumber = 96;

int   kLowestVelocity = 32;      // MIDI velocity (0-127)
int   kHighestVelocity = 110;

int   kLowestInstrument = 0;     // MIDI instrument (0-127)
int   kHighestInstrument = 127;

void setup()
{
  size(600, 400);
  colorMode(HSB, 128);

  MidiBus.list(); // List all available Midi devices on STDOUT. This will show each device's index and name.
  myBus = new MidiBus(this, "Bus 1", "Java Sound Synthesizer"); // Create a new MidiBus with no input device and the default Java Sound Synthesizer as the output device.
 
  smooth();
  
  background(0);
  noStroke();
  frameRate(8);
}

int lastNote = -1;

void draw()
{
  fill(0,0,0,10);
  rect(0,0,width,height);
  
  int instrument = (int) random(kLowestInstrument, kHighestInstrument);

  int noteNumber = (int) random(kLowestNoteNumber, kHighestNoteNumber);

  int velocity = (int) random(kLowestVelocity, kHighestVelocity);

  setInstrument( instrument );
  if (lastNote != -1) {
    myBus.sendNoteOff(kMidiChannel, lastNote, 0);
  }
  myBus.sendNoteOn(kMidiChannel, noteNumber, velocity);
  lastNote = noteNumber;

  fill( instrument, 90, 90 );
  ellipse( map(noteNumber,0,127,0,width),
           map(velocity,0,127,height,0),
           24, 16 );
}

void setInstrument(int inst)
{
   myBus.sendMessage(0xc0,kMidiChannel,inst,0);
}

void allNotesOff()
{
  for (int i = 0; i < 128; ++i)
    myBus.sendNoteOff(kMidiChannel, i, 0);
}

void keyPressed() 
{
  if (key == ' ') {
    isMute = !isMute;
    if (isMute) {
      allNotesOff();
    }
    println("MUTE " + (isMute? "ON" : "OFF"));
  }
}

void stop()
{
  allNotesOff();
  super.stop();
}

