/*******************************************************
Wheel of Stars

Processing Version

Copyright 2009 Jim Bumgardner

Version 1.1 Changed from rwMidi to MidiBus Library
Version 1.0 2009

********************************************************/
import themidibus.*;
import java.util.Vector;

MidiBus midiOutput;

final int kDefWidth = screen.width;
final int kDefHeight = screen.height;


int   kWidth = kDefWidth;             // width of graphics
int   kHeight = kDefHeight;            // height of graphics
int   kMidiChannel = 0;
int   kDefaultVelocity = 64;
int   kNoteDuration = 15 * 1000; // 8000
int   cx = kWidth/2,cy = kHeight/2;          // center coordinates
float[] tines;        // keeps track of current position of note, by angle
long[] lastSound;     // keeps track of time last note sounded
boolean[] isOn;
boolean isMute = false;
String lines[];

int maxMag = 8;
float siderealPeriod = (23*60*60 + 56*60 + 4.091);
float kSpeed = 2*PI/siderealPeriod;
float kDayPhase;

Vector  starList = new Vector();

// String kCatName = "hip_mag_7.txt";
String kCatName = "hip_mag_8_dec_minus_60.txt";  // Hipparcos Catalog
// String kCatName = "catalog.txt";     // Yale Bright Star Catalog
boolean kHipMode = true;  // Hipparcos format (otherwise Yale Bright Star format)

public class Star
{
  float  fMag, fDec, fRA, fBV;
  String buf;
  int    idx;
  
  public Star(String buf, int idx, float fMag, float fDec, float fRA, float fBV)
  {
      this.buf = buf;
      this.idx = idx;
      this.fMag = fMag;
      this.fDec = fDec;
      this.fRA = fRA;
      this.fBV = fBV;
  }
}
PrintWriter output;

void setup()
{
  size(kWidth,kHeight);
  // noLoop();
  smooth();
  colorMode(HSB,2);

  lines = loadStrings(kCatName);
  println("Loaded " + lines.length + " lines");
  int numStars = 0;
  kDayPhase = getDayPhase();
  
  // output = createWriter("starlist.as");  // Exported stars to actionscript array


  for (int i = 0; i < lines.length; ++i) {
    String lin = lines[i];
    // println(" line " + i);
    if (lin.length() < 10)
      continue;
    if (kHipMode) {
      if (!lin.substring(0,1).equals("H"))
        continue;
      String m_mag      = lin.substring(41, 41+5).trim();
      String m_ra       = lin.substring(51, 51+12).trim();
      String m_dec      = lin.substring(64, 64+12).trim();
      String c_bv       = lin.substring(245, 245+6).trim();
      if (m_ra.equals("") || m_dec.equals("") || m_mag.equals(""))
        continue;      
      if (m_dec.substring(0,1).equals("+"))
        m_dec = m_dec.substring(1);
      float fMag = Float.valueOf(m_mag);
      float dec = Float.valueOf(m_dec) * PI/180;
      float ra = Float.valueOf(m_ra) * PI/180;  // RA in radians
      float bv = c_bv.equals("")? 0 : Float.valueOf(c_bv);
      Star star = new Star("", numStars,fMag, dec, ra, bv);
      starList.add(star);
      // output.println("\t<i r=\"" + ra +"\" d=\""+dec+"\" m=\""+fMag+"\" b=\""+bv+"\"/>");
      // println("Added " + numStars);
    }
    else {  // YALE BRIGHT STAR MODE

      String m_mag      = lin.substring(102, 102+5).trim();
      
      if (m_mag.equals("") || Float.valueOf(m_mag) > maxMag)
        continue;
  
      
      String nom      = lin.substring(0, 4);
      
      String ra_hh    = lin.substring(75,75+2);
      String ra_mm    = lin.substring(77,77+2);
      String ra_ss    = lin.substring(79,79+4);
      
      String dec_sign = lin.substring(83, 83+1);
  
  
      String dec_deg  = lin.substring(84,84+2);
      String dec_mm   = lin.substring(86,86+2);
      String dec_ss   = lin.substring(88,88+2);
  
      // if (dec_sign.equals("-") && Float.valueOf(dec_deg) > 30)
      //  continue;
  
      //    if (dec_sign.equals("-"))
      //      continue;
      
      String g_lon    = lin.substring(90, 90+6);
      String g_lat    = lin.substring(96, 96+6);
      
      
      String c_bv     = lin.substring(109, 109+5);
      String c_ub     = lin.substring(115, 115+5);
      String c_ri     = lin.substring(121, 121+5);    
    
      float dec = Float.valueOf(dec_deg)+Float.valueOf(dec_mm)/60+Float.valueOf(dec_ss)/(60*60);
      if (dec_sign.equals("-"))
        dec *= -1;
      dec *= PI/180; // degrees to radians
      float fMag = Float.valueOf(m_mag);
      float ra = Float.valueOf(ra_hh) + Float.valueOf(ra_mm)/60 + Float.valueOf(ra_ss)/(60*60);
      // convert from hours to degrees
      // convert from hours to radians
      ra *= PI/12;
      float bv = c_bv.trim().equals("")? 0 : Float.valueOf(c_bv);
      // println("bv = " + bv);
  
  
      Star star = new Star(lin, numStars,fMag, dec, ra, bv);
      starList.add(star);
      // output.println("\t[" + ra +","+dec+","+fMag+","+bv+"],");
    }
    numStars++;
  }
  println(numStars + " stars");
  // output.flush();
  // output.close();
  
  tines = new float[numStars];
  lastSound = new long[numStars];
  isOn = new boolean[numStars];
  for (int i = 0; i < numStars; ++i)
  {
    tines[i] = -10;
    lastSound[i] = -30000;
    isOn[i] = false;
  }

  // Show available MIDI output devices here
  MidiBus.list(); // List all available Midi devices on STDOUT. This will show each device's index and name.
  midiOutput = new MidiBus(this, "Bus 1", "Java Sound Synthesizer"); // Create a new MidiBus with no input device and the default Java Sound Synthesizer as the output device.

  int stringEnsemble = 49;

  midiOutput.sendMessage(0xc0,0,stringEnsemble,0);
  midiOutput.sendMessage(0xc0,1,stringEnsemble,0);

  println("\nPress SPACE to MUTE (all notes off)");
  setPan(0,0);   // hard left pan for channel 0
  setPan(1,127); // hard right pan for channel 1
  
}

// Compute adjustment for local sidereal time, in radians
float getDayPhase()
{
    float lst = getLST();
    return (lst*2*PI/siderealPeriod);
}

float getLST()
{
    Date now = new Date();
    int gmC = 946728000;
    int n = (int) (now.getTime()/1000);
    float D = (n - gmC)/(24*60*60);
    float GMST = ( 18.697374558 + 24.06570982441908 * D );
    GMST /= 24.0;
    GMST -= Math.floor(GMST);
    GMST *= 24;
    TimeZone tz = TimeZone.getDefault();
    float tzOfst = tz.getOffset(now.getTime()) / (1000*60*60);
    println("Tz offset = " + tzOfst);
    float LST = GMST + tzOfst;
    if (LST < 0)  LST += 24;
    if (LST >= 24)  LST -= 24;
    println("LST = " + LST);
    return LST;
}

int panAssign(Star star, float px, float py)
{
   return (int) (px*128/width);
}

int pitchAssign(Star star)
{
//  if (reversePitches)
//    return (lowestNoteNumber+(nbrPoints-1)-i) % 128;
//  else
    float lowestPitch = 20;
    float highestPitch = 100;
    float lowBV = 1.5;
    float highBV = 0;

    float r = (star.fBV - lowBV) /  (highBV-lowBV);
    int pitch = (int) (r * (highestPitch - lowestPitch) + lowestPitch);
    pitch = max(0, min(pitch,127));
    // println("bv = " + star.fBV + " pitch = " + pitch);
    return pitch;
    // return ((int)(44 + star.fDec*80/90)) % 128;
}

int velAssign(Star star)
{
  int vel = (int) (127 / pow(1.5, star.fMag));
  // println("mag = " + star.fMag + " vel = " + vel);
  // return 127-(int)(star.fMag*15);
  return max(0,min(vel,127));
} 

void allNotesOff()
{
  for (int i = 0; i < 128; ++i) {
    midiOutput.sendNoteOff(kMidiChannel, i, 0);
    midiOutput.sendNoteOff(kMidiChannel+1, i, 0);
  }
}

void keyPressed() {
  if (key == ' ') 
  {
    isMute = !isMute;
    if (isMute) {
      allNotesOff();
    }
    println("MUTE " + (isMute? "ON" : "OFF"));
  }
}

void setPan(int chan, int value)
{
  midiOutput.sendControllerChange(chan,10,value);
}

void MyNoteOn(int n, int chan, int pitch, int vel)
{
  // println("Note On " + n + " : " + chan + " " + pitch + " " + vel);
  midiOutput.sendNoteOn(chan,pitch,vel);
}

void MyNoteOff(int chan, int pitch, int vel)
{
  // println("Note Off : " + chan + " " + pitch + " " + vel);
  midiOutput.sendNoteOff(chan,pitch,vel);
}

void draw()
{
  background(0);
  long cMillis = millis();
  float t = cMillis/1000.0;
  float pi2 = 2*PI;
  stroke(0,0,.25);
  noFill();
  line(0,height/2,width,height/2);
  int sz = starList.size();
  for (int i = 0; i < sz; ++i) {
    Star star = (Star) starList.elementAt(i);

    float d = (width/2) * (1- star.fDec/(PI*.5));
    if (star.fDec < 0) {
        d = (width/2) * (1+ abs(star.fDec)/(PI*.5));
    }
    float a = star.fRA;
    float magr = 8 / pow(1.5, star.fMag); // (1+maxMag) - Float.valueOf(star.fMag);
    float radv = max( magr+10-10*(cMillis-lastSound[i])/3000.0 , magr);
    float satv = min(.5, (cMillis-lastSound[i])/3000.0);
    noStroke();
    fill(1-star.fBV/2,min(satv*2,star.fBV),2);

    a -= (t*kSpeed + kDayPhase);
    float px = cy + cos(a)*d;
    float py = cy + sin(a)*d;
//    if (px < 0 || px > width || py < 0 || py > height)
//      continue;

    ellipse(px, py, radv, radv);

    if (cMillis > 15000 && 
        px >= 0 && px < width && py >= 0 && py < height &&
        Math.floor(a/PI) != Math.floor(tines[i]/PI))
    {
      // Sound Note Here...
      // println("nn: ang=" + a);
      if (!isMute) {
        // midiOutput.sendNoteOn(kMidiChannel, pitchAssign(numStars,dec,ra), velAssign(numStars,dec,ra,Float.valueOf(m_mag)));
        float rightMult = px > width/2? 1 : px/(width/2);
        float leftMult  = px < width/2? 1 : (px-width/2)/(width/2);
        // setPan( panAssign(star, px, py) );
        // Fake panning by playing both channels at different volumes...
        MyNoteOn( i, kMidiChannel, pitchAssign(star), int(velAssign(star)*leftMult) );
        MyNoteOn( i, kMidiChannel+1, pitchAssign(star), int(velAssign(star)*rightMult) );
        isOn[i] = true;
      }
      lastSound[i] = cMillis;
    }
    else if (isOn[i] && cMillis - lastSound[i] > kNoteDuration)
    {
        // midiOutput.sendNoteOff(kMidiChannel, pitchAssign(numStars,dec,ra), 0);
        MyNoteOff(kMidiChannel, pitchAssign(star), 0);
        MyNoteOff(kMidiChannel+1, pitchAssign(star), 0);
        isOn[i] = false;
    }
    tines[i] = a;
  }
  // filter(BLUR);
}

