// basic fft example - Jim Bumgardner
//
// Based on the example at http://code.compartmental.net/tools/minim/manual-fft/

import ddf.minim.*;
import ddf.minim.analysis.*;

Minim       minim;
AudioSource signal;
BeatDetect  beatDetector;
boolean     useInput = false;

void setup()
{
  size(500,500);
  minim = new Minim(this);

  // For BeatDetect, you'll get more 'timely' results with a 512 buffer size
  if (useInput) {
    AudioInput lineIn = minim.getLineIn(Minim.STEREO, 512);
    signal = (AudioSource) lineIn;
  } else {
    AudioPlayer player = minim.loadFile("554__bebeto__ambient-loop.mp3", 512);
    player.loop();
    signal = (AudioSource) player;
  }

  beatDetector = new BeatDetect(signal.bufferSize(), signal.sampleRate());
}


void draw()
{
  background(0);
  stroke(255);
  // perform a forward FFT on the samples in jingle's left buffer
  // note that if jingle were a MONO file, 
  // this would be the same as using jingle.right or jingle.left
  beatDetector.detect(signal.mix);

  if (beatDetector.isHat()) {
    fill(255);
    ellipse(width/4, height/4, width/9, width/9);
  }
  if (beatDetector.isSnare()) {
    fill(200);
    ellipse(width/2, height/2, width/9, width/9);
  }
  if (beatDetector.isKick()) {
    fill(150);
    ellipse(width*.75, height*.75, width/9, width/9);
  }
}

// Always include something like this when using minim
void stop()
{
  signal.close();
  minim.stop();
  super.stop();
}
