// basic fft example - Jim Bumgardner
//
// Based on the example at http://code.compartmental.net/tools/minim/manual-fft/



import ddf.minim.*;
import ddf.minim.analysis.*;

Minim       minim;
AudioSource signal;
boolean     useInput = false;

FFT         fft;

void setup()
{
  size(500,250);
  minim = new Minim(this);

  // Second param (optional) is buffer size in samples, default is 1024
  // This will affect how many buckets are created in FFT (generally half that
  // amount) - it should be a power of 2.
  if (useInput) {
    // Use iTunes or the Microphone
    AudioInput lineIn = minim.getLineIn(Minim.STEREO, 2048);
    signal = (AudioSource) lineIn;
  } else {
    // Play a file in the 'data' folder
    AudioPlayer player = minim.loadFile("554__bebeto__ambient-loop.mp3", 2048);
    // signal = minim.loadFile("77082__soundstack__streching-effect-with-balloon.wav",2048);
    player.loop();
    signal = (AudioSource) player;
  }


  // create an FFT object that has a time-domain buffer 
  // the same size as the sample buffer
  // note that this needs to be a power of two 
  fft = new FFT(signal.bufferSize(), signal.sampleRate());

 
}


void draw()
{
  background(0);
  stroke(255);
  // perform a forward FFT on the samples in jingle's left buffer
  // note that if jingle were a MONO file, 
  // this would be the same as using signal.right or signal.left
  fft.forward(signal.mix);
  for(int i = 0; i < fft.specSize(); i++)
  {
    // draw the line for frequency band i, scaling it by 4 so we can see it a bit better
    line(i, height, i, height - fft.getBand(i)*2);
  }
}

void keyReleased()
{
  if ( key == 'w' )
  {
    // a Hamming window can be used to shape the sample buffer that is passed to the FFT
    // this can reduce the amount of noise in the spectrum
    fft.window(FFT.HAMMING);
    println("HAMMING");
  }
 
  if ( key == 'e' )
  {
    fft.window(FFT.NONE);
    println("NONE");
  }
}

// Always include something like this when using minim
void stop()
{
  signal.close();
  minim.stop();
  super.stop();
}
