import processing.opengl.*;

// basic fft example - Jim Bumgardner
//
// Based on the example at http://code.compartmental.net/tools/minim/manual-fft/

import ddf.minim.*;
import ddf.minim.analysis.*;

Minim       minim;
AudioSource signal;

FFT         fft;

boolean useInput = true;
int     m = 1024;
float[] dots = new float[m+1];

void setup()
{
  size(512,512);
  minim = new Minim(this);

  if (useInput) {
      AudioInput lineIn = minim.getLineIn(Minim.STEREO, 2048);
      signal = (AudioSource) lineIn;
  } else {
    AudioPlayer player = minim.loadFile("554__bebeto__ambient-loop.mp3", 2048);
    // signal = minim.loadFile("77082__soundstack__streching-effect-with-balloon.wav",2048);
    player.loop();
    signal = (AudioSource) player;
  }

  fft = new FFT(signal.bufferSize(), signal.sampleRate());

   background(0);
   noStroke();
   println("specSize is " + fft.specSize());

  for (int i = 0; i <= m; ++i) {
    dots[i] = 0;
  }

}

float maxVal = 0;

void draw()
{
  copy(1,0,width-1,height,0,0,width-1,height);

  fft.forward(signal.mix);
  for(int i = 0; i < min(m, fft.specSize()); i++)
  {
    float v = fft.getBand(i);
    dots[i] += v/512;  // this array is used to retain some energy from pass to pass...
    dots[i] *= .9;

    float rv = constrain(dots[i],0,1);

    fill(rv*255);
    rect(width-2,(height-10)-i,2, 1);
  }
}

void keyReleased()
{
  if ( key == 'w' )
  {
    // a Hamming window can be used to shape the sample buffer that is passed to the FFT
    // this can reduce the amount of noise in the spectrum
    fft.window(FFT.HAMMING);
    println("HAMMING");
  }
 
  if ( key == 'e' )
  {
    fft.window(FFT.NONE);
    println("NONE");
  }
}

// Always include something like this when using minim
void stop()
{
  signal.close();
  minim.stop();
  super.stop();
}
