/**
  * This sketch demonstrates how to implement your own AudioSignal for Minim. 
  */
 
import ddf.minim.*;
import ddf.minim.signals.*;
 
Minim minim;
AudioOutput out;
SquareWave mosc;

void setup()
{
  size(512, 200, P3D);
 
  minim = new Minim(this);
 
  out = minim.getLineOut(Minim.STEREO, 2048, 44100);
  mosc = new SquareWave(440, .75, 44100);
  // adds the signal to the output
  out.addSignal(mosc);

  // mosc.portamento(50);
}
 
void draw()
{
  background(0);
  stroke(255);
  // draw the waveforms
  for(int i = 0; i < out.bufferSize()-1; i++)
  {
    float x1 = map(i, 0, out.bufferSize(), 0, width);
    float x2 = map(i+1, 0, out.bufferSize(), 0, width);
    line(x1, 50 + out.left.get(i)*50, x2, 50 + out.left.get(i+1)*50);
    line(x1, 150 + out.right.get(i)*50, x2, 150 + out.right.get(i+1)*50);
  }
}

float midiToFreq(float noteNbr)
{
  final float base_a4 = 440;
  return base_a4*pow(2,(noteNbr-57)/12.0);
}

void mouseMoved()
{
  // linear mapping
  // float freq = map(mouseX,0,width,440/4,440*4);

  // power of 2 mapping
  //
  // float freq = map(sq(mouseX/(float)width),0,1,440/4,440*4);

  // Midi pitch mapping
  float freq = midiToFreq( (int) map(mouseX,0,width,16,104) );


  float amp = map(mouseY,height,0,0,1);
  mosc.setFreq(freq);
  mosc.setAmp(amp);
}
 
void stop() // IMPORTANT
{
  out.close();
  minim.stop();
 
  super.stop();
}

