/* 
  How to filter a square wave and control filter with the mouse... 
  Sequence pitches from Pink Floyd: "On the Run"
  */
 
import ddf.minim.*;
import ddf.minim.signals.*;
import ddf.minim.effects.*;
import ddf.minim.analysis.*;

Minim minim;
AudioOutput out;
SquareWave  mosc;
BandPass    filt;
FFT         fft;

int[] sequence = {40,43,45,43,50,48,50,52};
float beats_per_minute = 165;
float beats_per_second = beats_per_minute/60;

int   notes_per_beat = 4;
float notesPerSec = notes_per_beat*beats_per_second;

void setup()
{
  size(500,500);

  frameRate(notesPerSec);
  
  minim = new Minim(this);
 
  int sampleRate = 44100;

  out = minim.getLineOut(Minim.STEREO, 2048, sampleRate);
  mosc = new SquareWave(440, .5, sampleRate);

  filt = new BandPass(440.0, 440.0, sampleRate);

  out.addEffect(filt);
  out.addSignal(mosc);

  fft = new FFT(out.bufferSize(), out.sampleRate());

}

int ctr;

void nextNote()
{
  ctr = (ctr + 1) % sequence.length;
  mosc.setFreq(midiToFreq(sequence[ctr]));
  mosc.setAmp(.75);
}
 
void draw()
{
  nextNote();

  background(0);
  stroke(255);
  // perform a forward FFT on the samples in jingle's left buffer
  // note that if jingle were a MONO file, 
  // this would be the same as using signal.right or signal.left
  fft.forward(out.mix);
  for(int i = 0; i < fft.specSize(); i++)
  {
    // draw the line for frequency band i, scaling it by 4 so we can see it a bit better
    line(i, height, i, height - fft.getBand(i));
  }
}

float midiToFreq(float noteNbr)
{
  final float base_a4 = 440;
  return base_a4*pow(2,(noteNbr-57)/12.0);
}

void mouseMoved()
{
  // linear mapping
  // float freq = map(mouseX,0,width,440/4,440*4);

  // power of 2 mapping
  //
  float freq = map(sq(mouseX/(float)width),0,1,440/4,440*5);

  float bwidth = map(sq(mouseY/(float)height),1,0,freq*.01,freq*10);

  filt.setFreq(freq);
  filt.setBandWidth(bwidth);
  println("Freq: " + freq + ", BWidth: " + bwidth);
}
 
void stop() // IMPORTANT
{
  out.close();
  minim.stop();
 
  super.stop();
}

