// fancy fft example - jbum
//
// Renders FFT analysis on a fibonacci spiral

import ddf.minim.*;
import ddf.minim.analysis.*;

int m = 256;     // number of circles
int sz = 600;    // width of screen

float cx = sz/2;
float cy = sz/2;

float rad = sz/2 * .95; // radius of rosette

float phi = (sqrt(5)+1)/2 - 1; // golden ratio
float ga = phi*2*PI;           // golden angle


Minim       minim;
AudioSource signal;
boolean     useInput = false;
FFT         fft;

float[] dots = new float[m+1];

void setup()
{
  size(sz,sz);
  minim = new Minim(this);

  if (useInput) {
      AudioInput lineIn = minim.getLineIn(Minim.STEREO, 512);
      signal = (AudioSource) lineIn;
  } else {
    AudioPlayer player = minim.loadFile("554__bebeto__ambient-loop.mp3", 512);
    // signal = minim.loadFile("77082__soundstack__streching-effect-with-balloon.wav",2048);
    player.loop();
    signal = (AudioSource) player;
  }

  fft = new FFT(signal.bufferSize(), signal.sampleRate());
  println("specSize is " + fft.specSize());

  noStroke();
  ellipseMode(CENTER);
  smooth();
  colorMode(HSB,1);
  for (int i = 0; i <= m; ++i) {
    dots[i] = 0;
  }
}

void draw()
{
  float maxArea = pow(rad,2)*PI;
  float maxRad = sqrt(maxArea/PI);
  float circArea = maxArea/m;
  float circRad = sqrt(circArea/PI);
  int   specSize = fft.specSize();
  
  background(0);
  fft.forward(signal.mix);
  for(int i = 0; i < min(m, specSize); i++)
  {
    float v = fft.getBand(i);

    dots[i] += v/specSize;  // this array is used to retain some energy from pass to pass...
    dots[i] *= .98;

    float rv = constrain(dots[i],0,1);
    fill(i/512.0, constrain(1-v/(specSize/4),0,1), rv);

    float a = i*ga; // angle
    float radcum = sqrt((circArea*i)/PI);
    float x = cx + cos(a)*radcum;
    float y = cy + sin(a)*radcum;
    ellipse(x,y,circRad*2*rv,circRad*2);

  }
}

// Always include something like this when using minim
void stop()
{
  signal.close();
  minim.stop();
  super.stop();
}
