// fancy fft example in 3d - Jim Bumgardner
//
// Renders FFT analysis on a spherical fibonacci spiral

import ddf.minim.*;
import ddf.minim.analysis.*;
import processing.opengl.*;

int nbrPoints = 1024;     // number of circles
float R = 277;
int sz = 900;    // width of screen

float cx = sz/2;
float cy = sz/2;

float rad = sz/2 * .95; // radius of rosette

float phi = (sqrt(5)+1)/2 - 1; // golden ratio
float ga = phi*2*PI;           // golden angle


Minim       minim;
AudioSource signal;
boolean     useInput = false;
FFT         fft;


float rotationX = 0;
float rotationY = 0;
float velocityX = 0;
float velocityY = 0;
float pushBack = 0;

class SpherePoint {
  float  lat,lon,e;
  SpherePoint(float lat, float lon)
  {
    this.lat = lat;
    this.lon = lon;
    this.e = 0; // energy
  }
};

SpherePoint[] pts = new SpherePoint[nbrPoints+1];

void setup()
{
  size(sz,sz,OPENGL);
  minim = new Minim(this);

  if (useInput) {
      AudioInput lineIn = minim.getLineIn(Minim.STEREO, 2048);
      signal = (AudioSource) lineIn;
  } else {
    AudioPlayer player = minim.loadFile("554__bebeto__ambient-loop.mp3", 2048);
    // signal = minim.loadFile("77082__soundstack__streching-effect-with-balloon.wav",2048);
    player.loop();
    signal = (AudioSource) player;
  }

  fft = new FFT(signal.bufferSize(), signal.sampleRate());
  println("specSize is " + fft.specSize());

  noStroke();
  ellipseMode(CENTER);
  smooth();
  colorMode(HSB,1);

  for (int i = 0; i < nbrPoints; ++i) {
    
    // Golden Tile Placement - this is close, but has pinching at poles
    float lon = ga*(i+1);
    lon /= 2*PI; lon -= floor(lon); lon *= 2*PI; 
    if (lon > PI)  lon -= 2*PI;

    float rat = i/(float)nbrPoints;
    float lat = -asin(-1 + 2*rat);

    pts[i] = new SpherePoint(lat, lon);
  }

  colorMode(HSB, 1);
  background(0);

}

void draw()
{
  fft.forward(signal.mix);
  int   specSize = fft.specSize();

  background(0);

  pushMatrix();
  translate(width/2.0, height/2.0, pushBack);
  lights();    
  pushMatrix();
  rotateX( radians(-rotationX) );  
  rotateY( radians(270 - rotationY) );
  strokeWeight(48);

  for(int i = 0; i < min(nbrPoints, specSize); i++)
  {
    float v = fft.getBand(i);

    pts[i].e += v/specSize;  // this array is used to retain some energy from pass to pass...
    pts[i].e *= .98;

    float rv = constrain(pts[i].e,0,1);
    stroke(i/512.0, constrain(1-v/(specSize/4),0,1), rv);
    // stroke(1,1,1);
    float lat = pts[i].lat; 
    float lon = pts[i].lon;

    pushMatrix();
    rotateY( lon);
    rotateZ( -lat);

    point(R,0,0);
      
    popMatrix();

  }
  popMatrix();  
  popMatrix();

  rotationY += 1/60.0;
  rotationX += 1/60.0;

  rotationX += velocityX;
  rotationY += velocityY;
  velocityX *= 0.95;
  velocityY *= 0.95;
  
  // Implements mouse control (interaction will be inverse when sphere is  upside down)
  if(mousePressed){
    velocityX += (mouseY-pmouseY) * 0.01;
    velocityY -= (mouseX-pmouseX) * 0.01;
  }

}

// Always include something like this when using minim
void stop()
{
  signal.close();
  minim.stop();
  super.stop();
}
