// Midibus Droplets - Jim Bumgardner 10/2/2011
//
// A recreation of my 'Hypnoticon' (circa 6/94)
//

import rwmidi.*; //Import the library

MidiOutput myOutput; // The MidiBus
MidiInput  myInput;

boolean midiThru = false;  // echo all received midi in messages to the midi out device

int  maxNotes = 128;

class MNote {
  long   startTime;
  float  x;
  float  y;
  float  initRad;
  float  weight;
  float  kMinVelocity = 10;
  float  kMaxVelocity = 100;
  int    pitch;
  int    velocity;
  
  boolean  isOn = false;

  MNote(int pitch) {
    this.pitch = pitch;
  }
  
  void init(int pitch, int velocity)
  {
    startTime = millis();
    this.pitch = pitch;
    this.velocity = velocity;
    this.x = random(width);
    this.y = random(height);
    this.initRad = 8 + velocity/128.0 * 20;
    this.weight = 2 + velocity/128.0 * 10;
    this.isOn = true;
  }
  
  void render()
  {
    if (!isOn)
      return;
    float elapsed = (millis() - startTime)*.001;
    float hueV = (this.pitch % 12) / 12.0;
    float lum = constrain(1-pow(elapsed/4,2), .05, 1);
    float sat = lum;
    float rad = initRad + elapsed*(kMinVelocity + (kMaxVelocity-kMinVelocity)*velocity/128.0);
    noFill();
    strokeWeight(weight);
    stroke(hueV,sat,lum);
    ellipse(x,y,rad*2,rad*2);
    // embossed shading
    stroke(hueV,sat,0,.5);
    ellipse(x+3,y+3,rad*2,rad*2);
  }
};


void setup() {
	size(screen.width,screen.height);
	background(0);

       println("INPUT");
       MidiInputDevice idevices[] = RWMidi.getInputDevices();
        for (int i = 0; i < idevices.length; i++) {
          println(i + ": " + idevices[i].getName());
        }

       println("OUTPUT");
       MidiOutputDevice odevices[] = RWMidi.getOutputDevices();
        for (int i = 0; i < odevices.length; i++) {
          println(i + ": " + odevices[i].getName());
        }
         
      

        // MODIFY THIS FOR YOUR MIDI SETUP
	//                 Parent  In        Out
	//                   |     |          |
       myOutput = odevices[2].createOutput();
       myInput = idevices[2].createInput();
       myInput.plug(this);

        colorMode(HSB, 1);
        smooth();
        noFill();

        for (int i = 0; i < maxNotes; ++i) {
          notes[i] = new MNote(i);
        }
}


MNote[] notes = new MNote[128];

void draw() {
    background(0);
    for (int i = 0; i < maxNotes; ++i) {
      notes[i].render();
    }
}

void noteOnReceived(Note note) {
	// Receive a noteOn
        int pitch = note.getPitch();
        int velocity = note.getVelocity();
        int channel = 0;
	println("Note On : ch=" +channel+" pitch="+pitch+" velocity="+velocity);
        if (midiThru) {
          myOutput.sendNoteOn(channel, pitch, velocity); // Send a Midi noteOn
        }
        notes[pitch].init(pitch, velocity);
}

void noteOffReceived(Note note) {
        int pitch = note.getPitch();
        int velocity = note.getVelocity();
        int channel = 0;
	// Receive a noteOff
	// println("Note Off: ch=" +channel+" pitch="+pitch+" velocity="+velocity);
        if (midiThru) {
          myOutput.sendNoteOff(channel, pitch, velocity); // Send a Midi noteOn
        }
        notes[pitch].isOn = false;
}

void programChangeReceived(ProgramChange pc)
{
}

void sysexReceived(SysexMessage sm) {
}

void controllerChangeReceived(Controller c) {
      int number = c.getCC();
      int value = c.getValue();
      int channel = 0;
	// Receive a controllerChange
	// println("Ctrl Ch : ch=" +channel+" number="+number+" value="+value);
        if (midiThru) {
          myOutput.sendController(channel, number, value); // Send a controllerChange
        }
}


